"""preparator contains EchoPreparator class."""


class EchoPreparator:
    """EchoPreparator returns input and output as it is."""

    @staticmethod
    def prepare_input(data):
        """prepare_input prepares input."""
        return data

    @staticmethod
    def prepare_output(data):
        """prepare_output prepares output."""
        return data
